#!/usr/bin/env python2

import VariantUtil
import subprocess
import numpy
import os
import sys
import logging
from H5Reader import H5Reader, StreamToLogger
from shutil import copyfile

def initChunk(input,interval):

    reader=H5Reader(input,0,20)
    filteredReadArray=[]
    readCount=0

    readsInInterval=reader.genome.readsInRange(reader.genomeName,interval[0],interval[1])
    for k, read in enumerate(readsInInterval):
        if read.MapQV >=reader.MQThreshold:
	    #print read
            filteredReadArray.append(read)
            readCount=readCount+1

    return filteredReadArray,readCount

def single_chunk_var_call(interval,input):

    filteredReadArray,readCount=initChunk(input,interval)
    return varcall((interval[0],interval[1]),readCount,filteredReadArray)

def multi_chunk_var_call(interval,input,alignmentIDs,tempdir,numberOfReads):

    logger = logging.getLogger('CallingVariants')

    if (alignmentIDs[interval[0]]==[]):
        logger.warn("NO ALIGNMENT for reading starting at " + str(interval[0]))
        return []

    tempname="tmp_"+str(interval[0])+".cmp.h5"
    temppath = os.path.join(tempdir, tempname)

    if(numberOfReads in alignmentIDs[interval[0]]):
        copyfile(input,temppath)
        filepath=temppath
    else:
        logger.info("Running cmph5tools select and sort for interval " + str(interval[0]) + '-' + str(interval[-1]))
        sortname="tmp_sorted"+str(interval[0])+".cmp.h5"
        filepath = os.path.join(tempdir, sortname)
        command="cmph5tools.py select " + input + " --outFile=" + temppath + " --idx "+''.join("{0} ".format(e) for e in alignmentIDs[interval[0]])
        sh_filename=os.path.join(tempdir,str(interval[0])+"-cmph5select.sh")
        temp_sh_file=open(sh_filename,"w")
        temp_sh_file.write(command)
        temp_sh_file.close()
        p1 = subprocess.call("sh "+ sh_filename,shell=True)
        # p1.wait()
        p2 = subprocess.call("cmph5tools.py sort " + temppath + " --outFile=" + filepath,shell=True)
        # p2.wait()
        os.remove(sh_filename)

    filteredReadArray,readCount=initChunk(filepath,interval)

    if(readCount==0):
        logger.warn("NO READS for read starting at " + str(interval[0]))
        os.remove(filepath)
        try:
            os.remove(temppath)
        except OSError:
            pass
        return []

    logger.info("Calling variants between " + str(interval[0]) + '-' + str(interval[-1]))

    variant_list=varcall(interval,readCount,filteredReadArray)

    logger.info("Finished calling variants between " + str(interval[0]) + '-' + str(interval[-1]))
    os.remove(filepath)
    try:
        os.remove(temppath)
    except OSError:
        pass

    # print "Number of variant in this region were:" + str(len(variant_list))
    return variant_list

def varcall(interval,readCount,filteredReadArray):
    read_index=0
    prevRef=''
    active_reads=[]
    variant_list=[]
    window_size=2
    deletion=False
    nextStart=filteredReadArray[0].referenceStart
    for position in range(interval[0],interval[1]):
        while(read_index<readCount):
            if(position >= nextStart):
                filteredReadArray[read_index].deletion=False
                active_reads.append(filteredReadArray[read_index])
                read_index=read_index+1
                if(read_index<readCount): nextStart=filteredReadArray[read_index].referenceStart
            else:
                break
        if(len(active_reads)==0): continue
        baseCallsList = {}
        qualityList={}
        toberemoved=[]
        for aln in active_reads:
            multi_del=False
            aligned = aln.clippedTo(position,position+window_size)
            if(aln.referenceEnd!=position+1):
                readlength=abs(aligned.tEnd-aligned.tStart)-1
                baseCall = aligned.read(orientation="genomic")[:-1]
            else:
                readlength=abs(aligned.tEnd-aligned.tStart)
                baseCall=aligned.read(orientation="genomic")
                toberemoved.append(aln)
            if baseCall == '':
                continue
            elif baseCall == "-":
                j=1
                if(aln.deletion):
                    multi_del=True
                else:
                    aln.deletion=True
                # TODO Might be infinite while loop
                b = True
                while b is True and (position + j) < interval[1]:
                    nextBase=aln.clippedTo(position+j,position+j+1)
                    if(nextBase.read(orientation="genomic") == "-"): multi_del=True
                    try:
                        baseCallQV = nextBase.pulseFeature("DeletionQV")[0]
                    except:
                        continue
                    if(baseCallQV != 255):
                        b = False
                    else: j=j+1
            else:
                aln.deletion=False
                try:
                    baseCallQV = numpy.mean(aligned.pulseFeature("QualityValue")[0:readlength])
                except:
                    continue

            if(baseCallQV==0 or baseCallQV==255): continue

            if(baseCall in baseCallsList):
                baseCallsList[baseCall]+=1
            else:
                baseCallsList.update({baseCall:1})
            if(baseCall in qualityList):
                qualityList[baseCall].append(baseCallQV)
            else:
                qualityList.update({baseCall:[baseCallQV]})
        for read in toberemoved: active_reads.remove(read)
        if(len(active_reads)==0):
            prevRef=''
            continue
        currentRef=active_reads[0].clippedTo(position,position+1).reference(orientation="genomic")

        concQual=VariantUtil.consensus_quality(qualityList)
        vcf_entry=VariantUtil.call_consensus(currentRef,position,baseCallsList,concQual,multi_del)
        if vcf_entry == None:
            # Adding a reset of the deletion variable to fix this error!!!!!!!!!!
            # For some reason, consensus_quality returns a None type object for one of the bases and
            # that reference is skipped
            deletion = False
            prevRef=currentRef
            continue
        if(vcf_entry.ALT=="-"):
            if(deletion):
                if(vcf_entry.FILTER!='LOW' and vcf_entry.FILTER!='NO'):
                    variant_list[-1].REF=variant_list[-1].REF+currentRef
                    variant_list[-1].RSR="NULL"
                    variant_list[-1].VSR="NULL"
                else:
                    vcf_entry.REF=prevRef+currentRef
                    vcf_entry.ALT=prevRef
                    variant_list.append(vcf_entry)
                    deletion=False
            else:
                vcf_entry.REF=prevRef+currentRef
                vcf_entry.ALT=prevRef
                variant_list.append(vcf_entry)
                if(vcf_entry.FILTER!='LOW' and vcf_entry.FILTER!='NO'):
                    deletion=True
        else:
            deletion=False
            variant_list.append(vcf_entry)

        prevRef=currentRef

    return variant_list


