#!/usr/bin/env python2

__author__ = 'yeonbinkim'

from pbcore.io import AlignmentSet
import logging


class H5Reader:

    def __init__(self, filename, interval, MQT):
        self.filename = filename
        self.genome = AlignmentSet(filename)
        self.genomelength = self.genome.refLengths.popitem()[1]
        self.genomeName = self.genome.refNames[0]
        self.intervalLength = interval
        self.MQThreshold = MQT
        self.Chunks = []
        self.interval = []
        self.alnIDs = {}
        self.numberOfReads = self.genome._length[0]

    def getReadsInterval(self):
        numberofchunks = self.genomelength / self.intervalLength + 1
        intervalPoints = []

        for i in range(1,numberofchunks):
            j=0
            advantage=False
            while True:
                readsAtCutPoint = self.genome.readsInRange(self.genomeName, self.intervalLength * i + j,
                                                           self.intervalLength * i + j + 1)
                for k, read in enumerate(readsAtCutPoint):
                    if read.clippedTo(self.intervalLength * i + j,
                                      self.intervalLength * i + j + 1).read(orientation=True) == "-":
                        j = j + 1
                        advantage = False
                        break;
                if advantage:
                    if i == 1:
                        intervalPoints.append((0, self.intervalLength * i + j))
                    else:
                        intervalPoints.append((intervalPoints[i-2][1], self.intervalLength * i + j))
                    break
                else:
                    advantage = True

        #intervalPoints.append((intervalPoints[-1][1] + 1,self.genomelength + 1))
        intervalPoints.append((intervalPoints[-1][1], self.genomelength))
        self.interval = intervalPoints

    def getChunkAlignmentIDs(self):

        for i in self.interval:
            first_id=0
            last_id=0
            readsInChunk = self.genome.readsInRange(self.genomeName, i[0], i[1])
        ######print "line 55, H5Reader.py, i[0] is: ", i[0]
        ######print "line 56, H5Reader.py, i[1] is: ", i[1]

            for k, read in enumerate(readsInChunk):
                if k == 0:
                    first_id=read.AlnID
                else:
                    last_id = read.AlnID

            #first_id=first_id-5 if first_id > 5 else 0
            #last_id=last_id+5 if last_id+5 < self.numberOfReads else self.numberOfReads-1
            first_id = first_id - 5 if first_id > 5 else 0
            last_id = last_id + 5 if last_id + 5 < self.numberOfReads else self.numberOfReads - 1
            self.alnIDs[i[0]] = range(first_id,last_id)


class StreamToLogger(object):
   """
   Fake file-like stream object that redirects writes to a logger instance.
   """
   def __init__(self, logger, log_level=logging.INFO):
      self.logger = logger
      self.log_level = log_level
      self.linebuf = ''

   def write(self, buf):
      for line in buf.rstrip().splitlines():
         self.logger.log(self.log_level, line.rstrip())

   def flush(self):
    pass

